// -----------------------------------------------------------------------------
/*
    ATTENZIONE!!! QUANDO IL PIC VIENE ALIMENTATO CON UNA TENSIONE INFERIORE AI 4 VOLT,
                  LA FREQUENZA NON DEVE SCENDERE SOTTO GLI 8 MHz.
                  SONO STATE RISCONTRATE ANOMALIE QUANDO LA FREQUENZA E' SUPERIORE.
                  IN PARTICOLARE GLI IN/OUT SONO GESTITI IN MODO ERRATO (USCITE CHE NON
                  COMMUTANO E RESTANO SEMPRE A '1').
*/
//------------------------------------------------------------------------------
/*
 * Project name:        MCP8901
                        Sonda temperatura I2C
                        Alimentazione: 2,7 - 5,5 V


 * Copyright:           (c) MikroElektronika, 2015.
                        JUMBO elettronica
                        jumboelettronica.altervista.org
                        jumboelettromania.altervista.org
                        jumboelettromania@gmail.com

  * Revision History:
                          05/05/2020 - initial release
                          10/05/2020 - Completa.

 * Description:         Primo funzionamento sonda temperatura basato su MCP8901
                        Progetto completo di programmazione T_set e T_hyst.
                        3 pulsanti e lcd Nokia 3310 completano l'opera.
                        T_set e T_hyst accettano come decimale il mezzo grado.
                        
                         First temperature probe operation based on MCP8901
 Complete project of T_set and T_hyst programming.
 3 buttons and Nokia 3310 lcd complete the work.
 T_set and T_hyst accept half degree as decimal.
 
                        EEPROM MAP: 0x00 + 0x01: T SET (0x00: integer, 0x01: decimal)
                                    0x02 + 0x03: T HYSTERES (0x02: integer, 0x03: decimal)
                                    0x04: CONFIG


 * Test configuration:
     MCU:             PIC16F1929
     Dev.Board:       Centrale Universale 2
     Schema:          Centrale Universale 2.sch
     Oscillator:      INT 8 MHz * 4 (PLL) = 32 MHz
     Ext. Modules:
     SW:              mikroC PRO for PIC

 * NOTES:

 */
 /*
 Mappatura pins del PIC16F1705
//  --- Pull-Up: tutti
 RA0 = pin 13
 RA1 = pin 12
 RA2 = pin 11
 RA3 = pin 4
 RA4 = pin 3
 RA5 = pin 2
//
 RC0 = pin 10
 RC1 = pin 9
 RC2 = pin 8
 RC3 = pin 7
 RC4 = pin 6
 RC5 = pin 5
//
 Vss = pin 14  ==> GND
 Vdd = pin 1 ==> +Vcc
********************************************************************************
Mappatura pins del PIC16F1709
// --- Pull-Up: tutti
 RA0 = pin 19
 RA1 = pin 18
 RA2 = pin 15
 RA3 = pin 4           PULS_6 (MCLR)
 RA4 = pin 3           PULS_5
 RA5 = pin 2
//
 RB4 = pin 13    LED_9   PULS_4      Morsettiera  TTF_LED  5-NOKIA_CLK
 RB5 = pin 12    LED_8   PULS_2      Morsettiera  TTF_SCK  4-NOKIA_DIN
 RB6 = pin 11    LED_7   PULS_3      Morsettiera  TTF_SDA  3-NOKIA_DC
 RB7 = pin 10    LED_6   PULS_1      Morsettiera  TTF_A0   2-NOKIA_CE
//
 RC0 = pin 16
 RC1 = pin 15    LED_11
 RC2 = pin 14    LED_10              Morsettiera            7-NOKIA_LED
 RC3 = pin 7     LED_3               Morsettiera
 RC4 = pin 6     LED_2               Morsettiera
 RC5 = pin 5     LED_1               Morsettiera
 RC6 = pin 8     LED_4               Morsettiera  TTF_CS
 RC7 = pin 9     LED_5               Morsettiera  TTF_RST   1-NOKIA_RST
//
 Vss = pin 20  ==> GND                                    8-NOKIA_GND
 Vdd = pin 1   ==> +Vcc                                   6-NOKIA_Vcc
*/
// **** Centrale Universale ****************************************************
// --- LED (OUTPUT)
 sbit LED_1 at LATC5_bit;   // OUT 1
 sbit LED_2 at LATC4_bit;   // OUT 2
 sbit LED_3 at LATC3_bit;   // OUT 3
 sbit LED_4 at LATC6_bit;   // OUT 4
 sbit LED_5 at LATC7_bit;   // OUT 5
 sbit LED_6 at LATB7_bit;   // OUT 6
 sbit LED_7 at LATB6_bit;   // OUT 7
 sbit LED_8 at LATB5_bit;   // OUT 8
 sbit LED_9 at LATB4_bit;   // OUT 9
 sbit LED_10 at LATC2_bit;   // OUT 10
 sbit LED_11 at LATC1_bit;   // OUT 11
// --- Pulsanti (INPUT)
 sbit pul_1 at RB7_bit;   // IN 1
 sbit pul_2 at RB5_bit;   // IN 2
 sbit pul_3 at RB6_bit;   // IN 3
 sbit pul_4 at RB4_bit;   // IN 4
 sbit pul_5 at RA4_bit;   // IN 5
 sbit pul_6 at RA3_bit;   // IN 6
// --- Ingressi analogici
 sbit ana_4 at RC0_bit;   // AN4 (Volt)
 sbit ana_5 at RC1_bit;   // AN5 (Amper)
// --- ESP8266
 sbit UART_Rx at RA1_bit;   // Rx (PIC) ==> Tx (ESP8266)
 sbit UART_Tx at RA0_bit;   // Tx (PIC) ==> Rx (ESP8266)
//
 sbit alert  at RA2_bit;    // Alert pin
// sbit clk  at LATA4_bit;
 //sbit ce  at LATA2_bit;

//
// sbit up at RB7_bit;   //
// sbit dw at RB5_bit;   //
// sbit set at RB4_bit;   //
//
 typedef struct {
        unsigned pul          : 1;
        unsigned half_set     : 1;  // half degree set
        unsigned half_hyst    : 1;  // half degree hysteresis
        unsigned mode         : 1;
        unsigned led          : 1;

 } Flags;
  Flags flag;
//
 unsigned char const jumbo [] ={"JUMBO elettr."};
 unsigned char const txt_set [] ={"Tset"};
 unsigned char const txt_hyst [] ={"Thys"};
 unsigned char const txt_on [] ={"ON"};
 unsigned char const txt_off [] ={"--"};
//
 unsigned char cnt_led;
 char txt[7], txt2[5];
 char txt32[11];
 //char cnt_pul;
 char buf;
 char ADC_ch;
 unsigned int temp16;
 unsigned int media_stack[32];
 char cnt_media = 0;
 unsigned int media;
 unsigned int data1, data2, data3, data4, data5, data6, data7, data8;
 unsigned int temperatura;
 char contr_set, contr_hyst, contr_mode;
 char t_set_l, t_hyst_l, t_set, t_hyst, t_mode;
 unsigned short t_set_h, t_hyst_h;
 char cent, dec, uni;
 char config = 0;
//
 #include "MCP8901_Lib.c"
 #include "Init_PIC16F182X.c"
 #include "Nokia3310_Lib_V2.c"
 #include "pulsanti.c"

//
// *****************************************************************************
//
// *****************************************************************************
 void Delay_msec (unsigned int count_delay) {
      for (count_delay; count_delay > 0; count_delay --) {
         Delay_ms (1) ;
          asm { clrwdt }
        }
 }
// *****************************************************************************
// temp_lsb
// *****************************************************************************
 char Calc_Decimale () {
  unsigned int buf;
//
      buf = 625 * (temp_lsb >> 4);
       return buf / 1000;
 }
// *****************************************************************************
// zero = 1: visualizza zeri iniziali
// *****************************************************************************
 void dec_to_str (char value, char zero) {
  char cnt = 0;
//
      if (value >= 0x80) {
       txt[0] = '-';                           // Segno "-"
        value -= 0x80;
      }
       else {
        txt[0] = (value / 100);                 // centinaia
         value = value - (txt[0]*100);
          txt[0] += 0x30;
       }
         txt[1] = value / 10;                  // decine
          txt[2] = (value - (txt[1] * 10)) + 0x30 ;        // unit
           txt[3]=0;
            if (txt[0] == 0x30 && zero == 0) txt[0] = ' ';
             //else txt[0] = txt[0] + 0x30;
              if (txt[1] == 0 && txt[0] == ' ' && zero == 0) txt[1] = ' ';
               else txt[1] = txt[1] + 0x30;
//
 }
// ***************************************************************************** MAIN PAGE bis
//
// *****************************************************************************
 void Main_Page_bis () {
  char buf;
// --- SET
     Write_String(txt_set, 4, 0);             //   "Tset"
      dec_to_str (t_set, 0);
       Write_String(txt, 4 + contr_set, 27);   // temperatura SET interi
        putchar(',', 0x00 + contr_set);              // ','
       //  putchar(t_set_l + 0x30, 0x00 + contr_set);  // temperatura SET decimle
         if (t_set_l >= 0x80) putchar ('5', 0x00 + contr_set);
          else putchar ('0', 0x00 + contr_set);
// --- Hyst
     Write_String(txt_hyst, 5, 0);              // "Thyst"
      dec_to_str (t_hyst, 0);
       Write_String(txt, 5 + contr_hyst, 27);   // Hysteresis (interi)
        putchar(',', 0x00 + contr_hyst);               // ','
      //   putchar(t_hyst_l + 0x30, 0x00 + contr_hyst);  // Hysteresis (decimale)
         if (t_hyst_l >= 0x80) putchar ('5', 0x00 + contr_hyst);
          else putchar ('0', 0x00 + contr_hyst);
// --- ALERT POLARITY MODE

     if (flag.mode) Write_Bitmap_page(mode_1, 5 + contr_mode, 59, 24);
      else Write_Bitmap_page(mode_0, 5 + contr_mode, 59, 24);
//
 }
// ***************************************************************************** MAIN PAGE Prog
// page.F7 = 1: scrive in contrasto.
// *****************************************************************************   //  Dec_BCD_Short (t_set_l, 4, 60, 0);
 void Main_Page_Prog () {
  char buf;
//
// --- SET
//
      if (t_set_h >= 0x80) {
        t_set = 255 - t_set_h;
         t_set |= 0x80;
      }
       else t_set = t_set_h;
//
      dec_to_str (t_set, 0);
       Write_String(txt, 4 + contr_set, 27);   // temperatura SET interi
        putchar(',', 0x00 + contr_set);              // ','
//
         if (flag.half_set == 0) {
           putchar('0', 0x00 + contr_set);  // temperatura SET decimle
         }
          else {
            putchar('5', 0x00 + contr_set);  // temperatura SET decimle
          }
// --- Hyst
//
      if (t_hyst_h >= 0x80) {
        t_hyst = 255 - t_hyst_h;
         t_hyst |= 0x80;
      }
       else t_hyst = t_hyst_h;
//
      dec_to_str (t_hyst, 0);
       Write_String(txt, 5 + contr_hyst, 27);   // Str, page, coll
        putchar(',', 0x00 + contr_hyst);
//
         if (flag.half_hyst == 0) {
           putchar('0', 0x00 + contr_hyst);  // temperatura SET decimle
         }
          else {
            putchar('5', 0x00 + contr_hyst);  // temperatura SET decimle
          }
// --- ALERT POLARITY MODE
// Mode 1: OUT ON quando la temperatura supera quella impostata
// Mode 1: OUT ON when the temperature exceeds the set one
            if (flag.mode) Write_Bitmap_page(mode_1, 5 + contr_mode, 59, 24);
// Mode 0: OUT ON quando la temperatura  inferiore a quella impostata
// Mode 0: OUT ON when the temperature is lower than the set one
             else Write_Bitmap_page(mode_0, 5 + contr_mode, 59, 24);
//
 }
// ***************************************************************************** MAIN PAGE
//
// Write_Double_String(char string[], unsigned char Page, unsigned char colonna, char pos_virgola)
//
// page.F7 = 1: scrive in contrasto.
// *****************************************************************************
 void Main_Page () {
  char buf;
//
     dec_to_str (temp_msb, 0);
      Write_Double_String(txt, 2, 0, 0);         // Interi
       txt[0] = ','; txt[1]= 0 ;
        Write_Double_String(txt, 2, 35, 0);      // ','
         buf = Calc_Decimale ();
          txt[0] = buf + 0x30;
           txt[1] = 0;
          Write_Double_String(txt, 2, 45, 0);    // Decimale
           txt[0] = ''; txt[1]= 'C'; txt[2]= 0;
            Write_Double_String(txt, 2, 60, 0);   // C
//
         if (alert) {
           Write_String(txt_on, 4, 70);
            LED_2 = 1;
         }
          else {
            Write_String(txt_off, 4, 70);
             LED_2 = 0;
          }
 }
// *****************************************************************************
// Programmazione
// Il pulsante "SET" cambia campo da aggiornare >> ora>> minuti>> data>> mese>> esce da programmazione
// I pulsanti "UP e "DW" incrementano o decrementano il valore in contrasto
// EEPROM MAP: 0x00 + 0x01: T SET
//             0x02 + 0x03: T HYSTERES
//             0x04: CONFIG
// *****************************************************************************
 void Program(){
  char cnt = 1;
//
     MCP8901_Read(2);       // T HYST
      t_hyst_h = temp_msb;
       t_hyst_l = temp_lsb;
        if (t_hyst_h >= 0x80) {
         t_hyst = 255 - t_hyst_h;
          t_hyst += 0x80;
        }
         else t_hyst = t_hyst_h;
          t_hyst_h = t_hyst;
           if (t_hyst_l >= 0x80) flag.half_hyst = 1;
            else flag.half_hyst = 0;
     MCP8901_Read(3);       // T SET
      t_set_h = temp_msb;
       t_set_l = temp_lsb;
        if (t_set_h >= 0x80) {
         t_set = 255 - t_set_h;
          t_set += 0x80;
        }
         else t_set = t_set_h;
          t_set_h = t_set;
           if (t_set_l >= 0x80) flag.half_set = 1;
            else flag.half_set = 0;
//
    while (1) {
     Puls();
      if (state_pul == 3) cnt ++;
//
       switch (cnt) {
            case (1): contr_set = 0x80;            // T SET
                      if (state_pul == 1) {
                       flag.half_set ^= 1;    // Button UP
                        if (flag.half_set == 0) {
                         t_set_l = 0;
                          t_set_h ++;
                           if (t_set_h == 126) t_set_h = 125;
                        }
                         else t_set_l = 0x80;
                      }
                        //
                        if (state_pul == 2) {
                         flag.half_set ^= 1;   // Button DW
                          if (flag.half_set == 1) {
                           t_set_l = 0x80;
                            t_set_h --;
                          }
                           else t_set_l = 0x00;
                          //
                            if (t_set_h == 199) t_set_h = 200;
                        }
                      break;
            case (2): contr_set = 0;                // T Hyst
                      contr_hyst = 0x80;
                      if (state_pul == 1) {
                       flag.half_hyst ^= 1;    // Button UP
                        if (flag.half_hyst == 0) {
                         t_hyst_l = 0;
                          t_hyst_h ++;
                           if (t_hyst_h == 126) t_hyst_h = 125;
                        }
                         else t_hyst_l = 0x80;
                      }
                        //
                        if (state_pul == 2) {
                         flag.half_hyst ^= 1;   // Button DW
                          if (flag.half_hyst == 1) {
                           t_hyst_l = 0x80;
                            t_hyst_h --;
                          }
                           else t_hyst_l = 0x00;
                          //
                            if (t_hyst_h == 199) t_hyst_h = 200;
                        }
                      break;
            case (3): contr_hyst = 0;                  // OUT Mode
                      contr_mode = 0x80;
                      if (state_pul == 1 || state_pul == 2) flag.mode ^= 1;
                      break;
            default: contr_mode = 0;           // Return
                     MCP8901_Write(2, t_hyst, t_hyst_l);   // Hysteresis
                      EEPROM_Write(0x02, t_hyst);          // Intger
                      EEPROM_Write(0x03, t_hyst_l);        // Decimal
                     MCP8901_Write(3, t_set, t_set_l);     // SET
                      EEPROM_Write(0x00, t_set);          // Intger
                      EEPROM_Write(0x01, t_set_l);        // Decimal
                     if (flag.mode) config.B2 = 1;
                      else config.B2 = 0;
                       EEPROM_Write(0x04, config);
                        Init_MCP8901 (config |= 0x60);    // Resolution 12 bit
                         Main_Page_Prog();
                      return;
       }
         Main_Page_Prog();
    }
 }
// *****************************************************************************
//
// *****************************************************************************
 char Test_EEprom(char addr){
  char buf;
//
    buf = EEPROM_Read(addr);
     if (buf != 0xFF) return buf;
      else return 0x00;
//
 }
// *****************************************************************************
//
// *****************************************************************************
 void interrupt(){
//
 }
// *****************************************************************************
//
// *****************************************************************************
 void main() {
  char ii=0,iii;
  unsigned short cnt_req;
//
     Init_PIC();
      Delay_msec(100);
       I2C_Init();
        config = Test_EEprom(0x04);        // Config
         Init_MCP8901 (config |= 0x60);    // Resolution 12 bit
          if (config.B4) flag.mode = 1;
           else flag.mode = 0;
          Delay_msec(500);
// --- Nokia GLCD
     initlcd();
// --- Nokia LED
      Nokia_LED =1;
       Delay_msec(100);
//
      Write_String(jumbo, 0, 5);
      contr_hyst =0;
       contr_set = 0;
        contr_mode = 0;
// --- Test
     //  MCP79410_Wr_All();
// --- Recupera Tset e Thyst     
    t_set_h = Test_EEprom(0x00);    // Integer SET
     t_set_l = Test_EEprom(0x01);    // Decimal SET
      MCP8901_Write(3, t_set_h, t_set_l);
    t_hyst_h = Test_EEprom(0x02);    // Integer HYSTERESIS
     t_hyst_l = Test_EEprom(0x03);    // Decimal HYSTERESIS
      MCP8901_Write(2, t_hyst_h, t_hyst_l);

     MCP8901_Read(3);       // T SET
      t_set_h = temp_msb;
       t_set_l = temp_lsb;
        t_set = t_set_h;
     MCP8901_Read(2);       // T HYST
      t_hyst_h = temp_msb;
       t_hyst_l = temp_lsb;
        t_hyst = t_hyst_h;
     Main_Page_bis ();

      LED_1 = 1;
// -----------------------------------------------------------------------------
    while (1) {
        if (INTCON.TMR0IF) {
         cnt_led ++;
          INTCON.TMR0IF = 0;
           cnt_req ++;
        }
         if (cnt_led >= 50) {
           LED_1 ^= 1;
          cnt_led = 0;
           flag.led ^= 1;
         }
// --- Gestione pulsanti
        Puls();
         if (state_pul == 3) Program();
// END pulsanti
//
        if (cnt_req>=120) {
          MCP8901_Read (0x00);
           Main_Page();
            cnt_req = 0;
        }
    }
 }
// *****************************************************************************
/*




























*/